package anywheresoftware.b4a.agraham.jmsgboxes;

import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.RaisesSynchronousEvents;
import jfxtras.labs.dialogs.MonologFX;
import jfxtras.labs.dialogs.MonologFXButton;

/**
 *This object provides various flavours of modal Msgbox.
 *A message box almost syntax compatible with the B4A Msgbox keyword may be shown by
 *
 *<code>Dim Msgbox As Msgboxes
 *...
 *Msgbox.Show("Message", "Title")</code> *
 *
 *A message box almost syntax compatible with the B4A Msgbox2 keyword may be shown by
 *
 *<code>Dim Msgbox As Msgboxes
 *...
 *Msgbox.Show2("Message", "Title", "Positive", "Cancel", "Negative")</code>*
 *
 *Note that the Basic4android Show2 Icon parameter is not present here
 *
 *A Msgbox with a customised set of buttons can be shown using Show3.
 */
@DependsOn(values = { "jfxtras-labs-2.2-r5" })
@ShortName("Msgboxes")
@Author("Andrew Graham")
@Version(1.2f)
public class Msgboxes
{
	//https://blogs.oracle.com/javajungle/entry/dialogfx_a_new_approach_to
	//https://blogs.oracle.com/javajungle/entry/monologfx_floss_javafx_dialogs_for
	//http://central.maven.org/maven2/org/jfxtras/jfxtras-labs
	
	// 1.0 initial release
	// 1.1	changed to using jfxtras-labs-2.2-r5.jar
	// 1.2	defined a DefaultButton(DialogResponse value) property
	//		defined Show3 with arbitrary type and number of buttons
	
	private final static double version = 1.2;
	
	
	// Default dialog type and button alignment
	private MonologFX.Type DlgType =  MonologFX.Type.INFO;	
	private MonologFX.ButtonAlignment BtnAlign =  MonologFX.ButtonAlignment.CENTER;
	private int BtnDefault = DialogResponses.NEGATIVE;
	private int x = -1, y = -1;
	
	
	/**
	 *This library contains a Msgboxes object that provides various flavours of modal Msgbox.
	 *
	 *This library uses the jfxtras-labs-2.2-r5.jar library from <link>JFXtras Labs|http://jfxtras.org</link>.
	 *This is the direct link to <link>jfxtras-labs-2.2-r5.jar|http://central.maven.org/maven2/org/jfxtras/jfxtras-labs/2.2-r5/jfxtras-labs-2.2-r5.jar</link>
	 *
	 *JFXtras and JFXtras Labs are licensed under the <link>New BSD License|http://en.wikipedia.org/wiki/BSD_licenses</link>.
	 *Copyright (c) 2013, JFXtras
	 *All rights reserved.
	 *
	 *This library is therefore also licensed under the New BSD License.
	 */
	public void LIBRARY_DOC()
	{
	}
	
	
	// Constructor
	
	public Msgboxes()
	{		
	}
	
	// Private methods
	
	/**
	 *Shows a modal message box with the specified message and title and a single OK button.
	 *Message - The dialog message.
	 *Title - The dialog title.
	 */
	private MonologFXButton.Type show(MonologFX dlg, String message, String title)
	{
		dlg.setMessage(message);
		dlg.setTitleText(title);
		dlg.setButtonAlignment(BtnAlign);
		dlg.setType(DlgType);
		if ((x > -1) && (y > -1))
			dlg.setPos(x, y);
		return dlg.showDialog();		
		
	}
		
	// Public methods
	
	@RaisesSynchronousEvents
	public void Show(String message, String title)
	{
		MonologFX dlg = new MonologFX();
		show(dlg, message, title);
	}
	
	/**
	 *Shows a modal message box with the specified message and title.
	 *Message - The dialog message.
	 *Title - The dialog title.
	 *Positive - The text to show for the "positive" button. Pass "" if you don't want to show the button.
	 *Cancel - The text to show for the "cancel" button. Pass "" if you don't want to show the button.
	 *Negative - The text to show for the "negative" button. Pass "" if you don't want to show the button.
	 *
	 *Returns one of the DialogResponse values corresponding to the type of the button pressed.
	 */
	@RaisesSynchronousEvents
	public int Show2(String message, String title, String positive, String cancel, String negative)
	{
		MonologFX dlg = new MonologFX();
		if (!negative.equals(""))
		{
			MonologFXButton btn = new MonologFXButton();
			btn.setLabel(negative);
			btn.setType(MonologFXButton.Type.NO);
			if (BtnDefault == DialogResponses.NEGATIVE)
				btn.setDefaultButton(true);
			dlg.addButton(btn);			
		}		
		if (!cancel.equals(""))
		{
			MonologFXButton btn = new MonologFXButton();
			btn.setLabel(cancel);
			btn.setType(MonologFXButton.Type.CANCEL);
			if (BtnDefault == DialogResponses.CANCEL)
				btn.setDefaultButton(true);
			dlg.addButton(btn);			
		}
		if (!positive.equals(""))
		{
			MonologFXButton btn = new MonologFXButton();
			btn.setLabel(positive);
			btn.setType(MonologFXButton.Type.YES);
			if (BtnDefault == DialogResponses.POSITIVE)
				btn.setDefaultButton(true);
			dlg.addButton(btn);
		}
		MonologFXButton.Type result = show(dlg, message, title);
		switch (result)
		{
			case YES : 
				return DialogResponses.POSITIVE;
			case NO : 
				return DialogResponses.NEGATIVE;
			case CANCEL :
				return DialogResponses.CANCEL;
			default:
				return DialogResponses.CANCEL; // should never get here		
		}
		
	}

	/**
	 *Shows a modal message box with the specified message and title and an arbitrary number and type of buttons.
	 *Message - The dialog message.
	 *Title - The dialog title.
	 *btnlabels - The text to show for each button in the array
	 *btntypes - The type for each button in the array as a dialog response value. This determines the DialogResponse returned for that button.
	 *
	 *Returns one of the DialogResponse values corresponding to the type of the button pressed.
	 */
	@RaisesSynchronousEvents
	public int Show3(String message, String title, String[] btnlabels, int[] btntypes)
	{
		MonologFX dlg = new MonologFX();
		for (int i = 0; i < btnlabels.length; i++)
		{
			MonologFXButton btn = new MonologFXButton();
			btn.setLabel(btnlabels[i]);
			switch (btntypes[i])
			{
				case DialogResponses.POSITIVE : 
					btn.setType(MonologFXButton.Type.YES); break;
				case DialogResponses.CANCEL : 
					btn.setType(MonologFXButton.Type.CANCEL); break;
				case DialogResponses.NEGATIVE : 
					btn.setType(MonologFXButton.Type.NO); break;
				case DialogResponses.ABORT : 
					btn.setType(MonologFXButton.Type.ABORT); break;
				case DialogResponses.RETRY : 
					btn.setType(MonologFXButton.Type.RETRY); break;
				case DialogResponses.IGNORE : 
					btn.setType(MonologFXButton.Type.IGNORE); break;
				case DialogResponses.CUSTOM1 : 
					btn.setType(MonologFXButton.Type.CUSTOM1); break;
				case DialogResponses.CUSTOM2 : 
					btn.setType(MonologFXButton.Type.CUSTOM2); break;
				case DialogResponses.CUSTOM3 : 
					btn.setType(MonologFXButton.Type.CUSTOM3); break;
				default:
					btn.setType(MonologFXButton.Type.CANCEL); break;
			}			
			if (BtnDefault == btntypes[i])
				btn.setDefaultButton(true);
			dlg.addButton(btn);				
		}
		MonologFXButton.Type result = show(dlg, message, title);
		switch (result)
		{
			case YES : 
				return DialogResponses.POSITIVE;
			case NO : 
				return DialogResponses.NEGATIVE;
			case CANCEL :
				return DialogResponses.CANCEL;
			case ABORT :
				return DialogResponses.ABORT;
			case RETRY :
				return DialogResponses.RETRY;
			case IGNORE :
				return DialogResponses.IGNORE;
			case CUSTOM1 :
				return DialogResponses.CUSTOM1;
			case CUSTOM2 :
				return DialogResponses.CUSTOM2;
			case CUSTOM3 :
				return DialogResponses.CUSTOM3;
			default:
				return DialogResponses.CANCEL; // should never get here		
		}		
	}
	
	
	/**
	 *By default the Msgbox is shown centred on the screen.
	 *If the values of both X and Y are >= 0 then the Msgbox will be positioned at that X, Y position.
	 *The positioning does seem to be off by about -3 pixels in both X and Y but that may depend on system settings for the windows border.
	 */
	public void SetXY(int X, int Y)
	{
		x = X; y = Y;
	}		

	
	// Public properties

	/**
	 *Returns the version number of the library.
	 */
	public double getVersion()
	{
		return version;
	}	
	
	/**
	 *Gets or sets the alignment of the buttons in the Msgbox
	 *Permissible values are "LEFT",  "RIGHT", or "CENTER"
	 *The library default is "CENTER"
	 */
	public void setButtonAlignment(String alignment)
	{
		BtnAlign = MonologFX.ButtonAlignment.valueOf(MonologFX.ButtonAlignment.class, alignment.toUpperCase());
	}	
	public String getButtonAlignment()
	{
		return BtnAlign.toString();
	}
	

	/**
	 *Gets or set a DialogResponses value that determines the type of the default button in the Msgbox.
	 *The library default is DialogResponses.NEGATIVE.
	 */
	public void setDefaultButton(int dialogresponse)
	{
		BtnDefault = dialogresponse;
	}
	public int getDefaultButton()
	{
		return BtnDefault;
	}
	
	
	/**
	 *Gets or sets the type of the Msgbox to be shown
	 *Permissible values are "ACCEPT",  "ERROR",  "INFO" or "QUESTION".
	 *The library default is "INFO"
	 */
	public void setDialogType(String dialogtype)
	{
		DlgType = MonologFX.Type.valueOf(MonologFX.Type.class, dialogtype.toUpperCase());
	}	
	public String getDialogType()
	{
		return DlgType.toString();
	}	
	
	

	

	/**
	 *The same dialog response values as Basic4android are implmemented. POSITIVE, CANCEL and NEGATIVE
	 *You will need to Dim a single Global of this type then you use it just like it was a keyword
	 *
	 *<code>Dim DialogResponse As DialogResponses</code>
	 *
	 *There are six additional button types/response values available for use with the Show3 method.
	 *ABORT, IGNORE, RETRY, CUSTOM1, CUSTOM2, and CUSTOM3
	 */
	@ShortName("DialogResponses")
	public static class DialogResponses
	{
		
		public static final int POSITIVE = -1;
		public static final int NEGATIVE = -2;
		public static final int CANCEL = -3;
		
		public static final int ABORT = -4;
		public static final int IGNORE = -5;
		public static final int RETRY = -6;
	  
		public static final int CUSTOM1 = -7;
		public static final int CUSTOM2 = -8;
		public static final int CUSTOM3 = -9;
	  
	  
	/**
	 *Return a string corresponding to the dialog response value provided or "UNKNOWN" if not one of the three allowed values.
	 */
	public String AsString(int dialogresponse)
	  {
		  switch (dialogresponse)
		  {
	  		case POSITIVE :
	  			return "POSITIVE";
	  		case CANCEL :
	  			return "CANCEL";
	  		case NEGATIVE :
	  			return "NEGATIVE";
	  		case ABORT :
	  			return "ABORT";
	  		case IGNORE :
	  			return "IGNORE";
	  		case RETRY :
	  			return "RETRY";
	  		case CUSTOM1 :
	  			return "CUSTOM1";
	  		case CUSTOM2 :
	  			return "CUSTOM2";
	  		case CUSTOM3 :
	  			return "CUSTOM3";	  			
	  		default : 
	  			return "UNKNOWN";
		  } 
	  }
	  
	}

}
